<?php

/*

 $Id: av.php 3544 2005-01-04 19:03:27Z philips $
 $HeadURL: https://svn.osuosl.org/public/maintain2/tags/release-2.4.2-RC2/modules/class/av.php $

 Copyright (c) 2002 Oregon State University - Network Engineering
 All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

 /**
 * av.php - defines the @see AV class 
 * @version $Id: av.php 3544 2005-01-04 19:03:27Z philips $
 * @copyright Copyright (c) 2004 Oregon State University - Network Engineering
 * @filesource
 */

/**
 *
 * Provides functions for getting AV information about hosts
 *
 * @author		Brandon Philips @version		
 * @package modules
 */

class AV {
	/**
	 * Host that this AV info belongs to as an object
	 */
	var $host;
	/**
	 * Unix timestamp, date of pattern files
	 */
	var $pattern_date;
	/**
	 * Pattern file is "out of date", calculated using a setting out_of_date set
	 *  in the module config
	 */
	var $out_of_date;
	var $pattern_version;
	var $file_revision;
	var $last_virus;
	var $last_scan;

	function AV ($id=0) { 
		if ($id != 0) {
			// If needed a by host constructor could go here.				
		}
	}

	/**
	*
	* Gets all information necessary for dispalying AV information on hosts in
	* the zone
	*
	* @return array  Returns an array of @see AV objects ordered by 
	* 'patternFileDate' by default
	*
	* @param integer $id The zone_id of hosts to get the AV information for
	* @param string $orderby The field in the table to order by
	*
	*/
	function get_by_zone($id, $orderby='patternFileDate', $order) { 
	
		global $conf;

		$zone = new Zone($id);

		$hosts = $zone->hosts();
		
		return AV::factory($hosts, $orderby);
	}

	function is_out_of_date($date, $limit) {
		$this->out_of_date = false;

		if ((time() - $date) > $limit) {
			$this->out_of_date = true;
		}

		return $this->out_of_date;
	}	

	/**
	 * Sets the current objects fields using an assoc array from a select on the
	 * av_info table
	 *
	 * @param array $i Assoc array from the av_info table @see mysql_fetch_assoc
	 * @param array $av_info The av_info modules info
	 */
	function set_from($i, $av_info) {
		$this->host = new Host($i['hostname']);
		$this->pattern_date = $i['UNIX_TIMESTAMP(patternFileDate)'];
		$this->out_of_date = AV::is_out_of_date($this->pattern_date, $av_info['out_of_date']);
		$this->pattern_version = $i['patternVersion'];
		$this->product_version = $i['productVersion'];
		$this->file_revision = $i['patternFileRevision'];
		$this->last_virus = $i['UNIX_TIMESTAMP(timeOfLastVirus)'];
		$this->last_scan = $i['UNIX_TIMESTAMP(timeOfLastScan)'];
	}

	/**
	 * Creates an array of AV objects from an array of host object.
	 * 
	 * @param object $hosts Host objects to grab
	 * @param string $orderby The field in the table to order by
	 *
	 * @returns array An array of AV objects
	 */
	function factory($hosts, $orderby, $order="DESC") {
		global $conf;

		if (!count($hosts)) {
			return array();
		}

		// Translate from the objects field name to the database field name
		switch ($orderby) {
		case pattern_version: 
			$orderby = patternVersion;
			break;
		case product_version: 
			$orderby = productVersion;
			break;
		case file_revision:
			$orderby = patternFileRevision;
			break;
		case last_virus: 
			$orderby = timeOfLastVirus;
			break;
		case last_scan:
			$orderby = timeOfLastScan;
			break;
		default:
			$orderby = patternFileDate;
		}

		if (!$order) { $order = "DESC"; }

		$info = $conf['modules']->get_info("av");
		$avdb = $info['db_name'];
		$avtable = $info['db_table'];

		$ldb = $conf['local_db'];

		$sql = "SELECT hostname,
				UNIX_TIMESTAMP(patternFileDate),
				patternVersion,
				patternFileRevision,
				UNIX_TIMESTAMP(timeOfLastVirus),
				productVersion,
				UNIX_TIMESTAMP(timeOfLastScan)
				FROM $avtable
				WHERE ( ";
		// Limit by hosts in this zone only
		foreach ($hosts as $host) {
			// Ensure that we are past the first element before putting on the ORs
			if (isset($pastfirst)) { $sql .= " OR "; } 
			// Select only hosts from this zone.
			$id = $host->id;
			$sql .= "hostname=$id";
			// We are now past the first element, start adding in the OR
			$pastfirst = true;
		}
		$sql .= " ) ORDER BY $orderby $order";
		
		$db_results = mysql_query($sql, $info['dbh']);

		if (!$db_results) {
			die('Invalid query: ' . mysql_error());
		}
		
		$avs = array();

		// Create an array of AV objects from the results
		while ($i = mysql_fetch_assoc($db_results)) {
			$av = new AV();
			$av->set_from($i, $info);
			$avs[] = $av;
		}

		return $avs;
	}	

	function make_href($name, $field, $zone_id) {
		global $conf;

		return "<a href={$conf['script']}?orderby=$field&amp;zone_id=$zone_id>$name</a>";
	}

	function get_current_pattern_date($format=0) {
		global $conf;

		$info = $conf['modules']->get_info("av");
		$avdb = $info['db_name'];
		$deftable = $info['db_current_def_table'];
	
		$sql = "SELECT UNIX_TIMESTAMP(date) FROM $deftable";
		$db_results = mysql_query($sql, $info['dbh']);

		if (!$db_results) {
			die('Invalid query: ' . mysql_error());
		}

		$date = mysql_fetch_row($db_results);

		if ($format)
			$date = date($format, $date[0]);
		else
			$date = $date[0];

		return $date;
	}

	function get_current_revision($format=0) {
		global $conf;

		$info = $conf['modules']->get_info("av");
		$avdb = $info['db_name'];
		$deftable = $info['db_current_def_table'];
	
		$sql = "SELECT revision FROM $deftable";
		$db_results = mysql_query($sql, $info['dbh']);

		if (!$db_results) {
			die('Invalid query: ' . mysql_error());
		}

		$revision = mysql_fetch_row($db_results);

		return $revision[0];
	}

}

