<?php

/*

 $Id: lib.php 3561 2005-02-01 00:42:01Z philips $
 $HeadURL: https://svn.osuosl.org/public/maintain2/tags/release-2.4.2-RC2/modules/lib.php $

 Copyright (c) 2002 Oregon State University - Network Engineering
 All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*

 These are the supporting functions for Maintain

*/

// This function is used to setup the user environment whenever
//    they visit a page ... it is intended to be run after check_session
//    and before any work actually happens.

/**
*
* Setup the users "environment"
*
* This function has alot of side effects that can best be summed up in the code
* but a mostly complete list follows:
*
* - Creates the global user object from $_SESSION
* - Sets the users colors
* - Checks for autoreg
* - Checks the gone.fishing file
* - Decide the zone the user wants to be viewing
* - Creates global $conf with ui, error and help objects
* - Shows the cancelled action template if need be
*
* @access	public
*/
function setup_user_environment ($action = false, $zone_id = 0) {
	global $conf, $user;

	// First create the user object from $_SESSION['user']
	$user = $_SESSION['user'];
	if (get_class($user) != 'user') { echo "<p class=\"fatalerror\" align=\"center\">Error: Your session has had something very bad happen to it, please <a href=\"" . $conf['web_path'] . "/logout.php\">log out</a> and log back in."; exit(); }
	$user->set_from_db();
	
	$conf['color'] = new Color($user->color_template);
	
	// If they are autoreg'in and they've set the autoreg_type
	if ($conf['modules']->modules['autoreg']->enabled) {
		if ($_REQUEST['autoreg_type']) { 
			$user->autoreg_type = $_REQUEST['autoreg_type']; 
		} 
	}

	// first thing's first, check for the gone fishing file
	if ( file_exists($conf['prefix'] . "/build/gone.fishing") ) {
		show_template('gonefishing');
		exit();
	}

	// choose a zone here
	if ( $_REQUEST['action'] == 'Go!' && $_REQUEST['zone_id'] && $user->has_read_access($_REQUEST['zone_id']) ) {
		$conf['zone'] = new Zone($_REQUEST['zone_id']);

		//check and reset if they are a zone admin or not.
		$zone_id = $conf['zone']->id;
	
		$user->current_zone = $_REQUEST['zone_id'];

		//sets is_admin,is_zone_admin,is_user,is_guest
		$user->set_access_level($_REQUEST['zone_id']);

		$user->view = false;

	}

	elseif ( $user->current_zone ) {
		$conf['zone'] = new Zone($user->current_zone);
	}
	elseif ( $user->preferred_zone AND $user->has_read_access($user->preferred_zone) ) {
		$conf['zone'] = new Zone($user->preferred_zone);
		$user->set_access_level($conf['zone']->id);
	}
	else {
		$zones = $user->zones();
		$conf['zone'] = new Zone($zones[0]->id);
		$user->set_access_level($conf['zone']->id);
	}

	$conf['action'] = $_REQUEST['action'];
	$conf['script'] = $SCRIPT_NAME;

	// if they don't specify an action we want to clear the user view
	//    so it doesn't inherit the current view for a different list.
	if ( ! $_REQUEST['action'] ) {
		$user->view  = false;
	}

	// let's make our other objects we'll need here as well
	$conf['ui']    = new UI();
	$conf['error'] = new Error();
	$conf['help']  = new Help();

	// we have established this person as a real user
	$conf['max_list'] = $user->list_view;

	//If they have canceled something then redirect off to a template
	if ( $action == 'Cancel' ) {
		require $conf['prefix'] . "/templates/show_cancel.inc";
		exit();
	} //cancel

}


/**
*
* Shows a template from /templates/
*
* @access	public
* @param	$template	the template to include (without .inc) so,
*						the function really does a 
*						require "/templates/$template.inc"
*/
function show_template ( $template ) {
        global $conf, $user;

        require $conf['prefix'] . "/templates/$template.inc";
}

/**
*
* Returns the global Maintain database handler
*
* Also does some error handling if the dbh is no longer a resource.
*
* @return	resource the global database connection for maintain
* @access	public
* @see		mysql_connect()
*/
function dbh() {
	global $conf;

	if (!is_resource($conf['dbh'])) { 
		echo "Fatal Error: Database Connection Failed. Please check your Maintain database.";
	}

	return $conf['dbh'];
	
} // dbh


/*
 Pull all the IP ranges from IP_blocks and then slowly count through them
 Creating empty ranges
*/
function get_available_ranges () {
	global $conf;

	//Get IP Blocks
	$sql = "SELECT * FROM ip_blocks WHERE enabled";
	$db_results = mysql_query($sql, $conf['dbh']);

	while ($r = mysql_fetch_object($db_results)) {
		$ip_blocks[] = $r;
	}

	$ip = new IP(0);

	//go through all ip blocks
	foreach ($ip_blocks as $ip_block) { 

		$ip_block_count = $ip_block->start;
		$range_start = $ip_block_count;
	
		while ($ip_block->end >= $ip_block_count) {

			$range_end = $ip_block_count;

			$sql = "SELECT end FROM ranges WHERE start='$ip_block_count'";	
			$db_results = mysql_query($sql, $conf['dbh']);

			//if there is a range there.... 
			//reset range_start and set ip_block_count to +1 end of range
			if ($r = mysql_fetch_object($db_results)) {

				//if they has been at least 2 empty IP
				//create a new range object manualy
				if ($range_start != $range_end-1 AND $range_start+1 != $range_end-1) { 
					$range->start_int = $range_start;
					$range->start     = $ip->int2ip($range_start);

					//the end is current - 1 (we found a range on this one so get the one right before
					$range->end_int = $range_end-1;
					$range->end     = $ip->int2ip($range_end);
		
					//set the "total ips" in this range
					$range->total = ($range_end - 1) - $range_start;

					//add it to the ranges array
					$ranges[] = $range;
					
				} //end start != end and start+1 != end
				
				//now reset the vars to the end of the found range account for the subnet
				//we have to account for the fact that end+1 will be the gateway address
				$range_start = $r->end + 2;
				$ip_block_count = $r->end+2;

			} //if something is found

			$ip_block_count++;

		} //end while inside ip_block


	} //foreach ip_blocks

	return $ranges;


} // get_available_ranges


// see get_available
function get_available_subnets () {

        global $conf;

        //Get IP Blocks
        $sql = "SELECT * FROM ip_blocks WHERE enabled='true'";   
        $db_results = mysql_query($sql, $conf['dbh']);

        while ($r = mysql_fetch_object($db_results)) {
                $ip_blocks[] = $r;
        }

        $ip = new IP(0);

        //go through all ip blocks
        foreach ($ip_blocks as $ip_block) {

                $ip_block_count = $ip_block->start;
                $subnet_start = $ip_block_count;

		//Make this here we will need it for later
		$max_netmask = $ip->ip2int('255.255.255.255');

                while ($ip_block->end >= $ip_block_count) {

                        $subnet_end = $ip_block_count;

                        $sql = "SELECT subnet,netmask FROM subnet WHERE subnet='$ip_block_count'";
                        $db_results = mysql_query($sql, $conf['dbh']);

                        //if there is a range there....
                        //reset range_start and set ip_block_count to +1 end of range
                        if ($r = mysql_fetch_object($db_results)) {

                                //if they has been at least 1 empty IP
                                //create a new range object manualy
                                if ($subnet_start != $subnet_end) {
					//we add 1 so it includes each end.. 1 to 5 is 5 possible not 4
                                        $subnet->start     = $ip->int2ip($subnet_start);
					$diff = $subnet_end - ( $subnet_start + 1 );
				
					$netmask = $max_netmask - $diff;

                                        //now current get the one right before
                                        $subnet->netmask = $ip->int2ip($netmask);

                                        $subnets[] = $subnet;

                                } //end start != end

				//get the total number of ip's in said subnet
				$diff = $max_netmask - $r->netmask;
				
                                //now reset the vars to the end of the found range account for the subnet
                                $subnet_start = $r->subnet + $diff + 1;
                                $ip_block_count = $subnet_start - 1;

                        } //if something is found

                        $ip_block_count++;

                } //end while inside ip_block


        } //foreach ip_blocks

        return $subnets;

} // get_available_subnets


/**
*
* Parse out the various bits of a hostname
*
* @return	array		Returns an associative array with the following keys:
*						- name, the 'hostname' ex. abe.osuosl.org, name == abe
*						- domain, the full domain ex. osuosl.org
*						- tld, top level domain ex. org
* @access	public
*/
function split_hostname($hostname) {
	global $conf, $user;

	//first we need to break out the "NAME" this should be everything before the first dot
	$raw_data = preg_split("/\./",$hostname);
	$object->name = $raw_data[0];

	//now let's get everything after the first dot. This should be the domain.
	//so a substr starting at the postition of the first dot +1 (we don't want the dot)
	$object->domain = substr($hostname, (strpos($hostname,".")+1),strlen($hostname));

	// Figure out what the hosts top level domain is
	$object->tld = $raw_data[count($raw_data) - 1];

	//we should be done return the object 
	return $object;

} //split_hostname

// count functions
	// count_cname, find total cnames or cnames per zone
	function count_cname($zone_id = 0) {
		global $conf;

	        if ($zone_id) {
		        $sql = "SELECT id FROM zone_cname WHERE zone='$zone_id'";
	        }
	        else {
			$sql = "SELECT id FROM zone_cname";
	        }

                $db_results = mysql_query($sql, $conf['dbh']) or Die (mysql_error());
                $count = mysql_num_rows($db_results);

        return $count;

        } // count_cname

	/*!
		@function count_pointer
		@discussion count all pointers in $zone_id or all if no zone specified
	*/
	function count_pointer($zone_id = 0) {
		global $conf;

		if ($zone_id) { 
			$sql = "SELECT id FROM pointer WHERE zone='$zone_id'";
		}
		else {
			$sql = "SELECT id FROM pointer";
		}

		$db_results = mysql_query($sql, $conf['dbh']);
		$count = mysql_num_rows($db_results);

	return $count;

	} // count_pointer

	/*!
		@function count_ranges
		@discussion give the total number of ranges in maintain
	*/
	function count_ranges() {
		global $conf;

		$sql = "SELECT count(id) FROM ranges";
		$db_results = mysql_query($sql, $conf['dbh']);
		
		$results = mysql_fetch_row($db_results);

		return $results[0];

	} // count_ranges

	// count_mx, either find the total mx records or mx per zone
	function count_mx($zone_id = 0)
	{
		global $conf, $user;

		if ($zone_id)
		{
		$sql = "SELECT id FROM zone_mx WHERE zone=" . $zone_id;
		}
		else
		{
		$sql = "SELECT id FROM zone_mx";
		}
		$db_results = mysql_query($sql, $conf['dbh']);
		$count = mysql_num_rows($db_results);

	return $count;

	} // count_mx
	
        // count_hosts, either find total hosts or hosts per zone
        function count_host($zone_id = 0)
        {
                global $conf, $user;

                if ($zone_id)
                {
                $sql = "SELECT count(*) FROM host WHERE zone='$zone_id'";
                $db_results = mysql_query($sql, $conf['dbh']);
		$count = mysql_fetch_row($db_results);
                }
                else
                {
                $sql = "SELECT count(*) FROM host";
                $db_results = mysql_query($sql, $conf['dbh']);
                $count = mysql_fetch_row($db_results);
                }
        return $count[0];

        } // count_host

	//give me a count of all of the zones
function count_zone($zone_id = 0) 
{
	global $conf, $user;

        // if we are only looking for my zones then...
        if ($zone_id) {
	        $zones = $user->zones();
                $count = count($zones);
        }
        // otherwise were looking for total zones
        else {
        $sql = "SELECT id FROM zone ";
        $db_results = mysql_query($sql, $conf['dbh']);
        $count = mysql_num_rows($db_results);
        }

return $count;

} // count_zone

        //counts the current number of active users of maint
        function count_user()
        {
                global $conf,$user;

                $sql = "SELECT username FROM user";
                $db_results = mysql_query($sql, $conf['dbh']);
                $count = mysql_num_rows($db_results);
                return $count;
        } // count_user

        // function to count the number of domains
        function count_domain ($zone_id = 0)
        {
                global $conf, $user;

                // Only domains in current zone
                if ($zone_id)
                {
		$sql = "SELECT count(*) FROM zone_domain WHERE zone='$zone_id'";
		$db_results = mysql_query($sql, $conf['dbh']);
		$count = mysql_fetch_row($db_results);
                }
                else {
                $sql = "SELECT count(*) FROM domain";
                $db_results = mysql_query($sql, $conf['dbh']);
                $count = mysql_fetch_row($db_results);
                }

        return $count[0];
        } // count_domain

	function count_dynamic_ip () {
		global $conf;

		$sql = "SELECT count(*) FROM host WHERE dynamic_range>'0' AND ip='0'";
		$db_result = mysql_query($sql, $conf['dbh']);

		$count = mysql_fetch_array($db_result);
		return $count[0];
	}

	function count_static_ip () {
		global $conf;

		$sql = "SELECT count(*) FROM host WHERE ip>'0' AND dynamic_range='0'";
		$db_result = mysql_query($sql, $conf['dbh']);

		$count = mysql_fetch_array($db_result);
		return $count[0];
	}

	function count_free_ip () {

	}

function count_soa () {
	global $conf;

	$sql = "SELECT count(*) FROM soa";
	$db_result = mysql_query($sql, $conf['dbh']);
	$count = mysql_fetch_row($db_result);

	return $count[0];
}

// Several functions that don't have a home anywhere else


// $domain is domain_id and server is a string (fqdn of nameserver)
//    This is usually used for delegations (see insert_nameserver in
//    class SOA for adding nameservers to SOA's)
function insert_nameserver ( $domain, $server ) {
	global $conf;

	$sql = "INSERT INTO record_ns (name,domain) VALUES ('$server', $domain)";
	$db_result = mysql_query($sql, $conf['dbh']);

	return mysql_insert_id($conf['dbh']);
}

function insert_soa ($domain, $primary_master, $mail_address, $refresh, $retry, $expire, $ttl) {
	global $conf;

	$sql = "INSERT INTO soa (domain,primary_master,mail_address,refresh,retry,expire,ttl) ".
		"VALUES ($domain, '$primary_master', '$mail_address', '$refresh', '$retry', '$expire', '$ttl')";
	$db_result = mysql_query($sql, $conf['dbh']);

	return mysql_insert_id($conf['dbh']);
}

// This will return an array of all SOA objects in the database
//
function get_soa () {
	global $user, $conf;

	$sql = "SELECT id FROM soa";

	if ( $user->view ) {
		$sql .= $user->view->get_query();
	}

	$db_result = mysql_query($sql, $conf['dbh']);

	$soas = array();

	while ( $soa_id = mysql_fetch_object($db_result) ) {
		$soa = new SOA($soa_id->id);
		$soas[] = $soa;
	}

	return $soas;
}

// This will return an array of all Domain objects in the database
//   that aren't associated with an SOA (and hence delegated)
function get_delegated_domains () {
	global $conf;

	$sql = "SELECT DISTINCT record_ns.domain FROM record_ns LEFT JOIN soa ON".
		" record_ns.domain=soa.domain WHERE soa.id IS NULL";
	$db_result = mysql_query($sql, $conf['dbh']);

	$domains = array();

	while ( $domain_obj = mysql_fetch_object($db_result) ) {
		$domain = new Domain($ns_obj->domain);
		$domains[] = $domain;
	}

	return $domains;
}

// This will build a CSV file with all of the hosts passed to it
//   and then mail it off to the named user.
function deliver_csv ($email, $zone, $hosts=0,$cnames=0,$mxs=0,$flos=0) {
	global $conf;


	//if the hosts is set send a host report
	if ($hosts) { 
	
	// generate mail attachment line
	$border_random = md5(time());
	$mail_boundary = "x{$border_random}x";

	$mail_headers = "From: $email\r\n";
	$mail_headers .= "X-Mailer: NET Mail Gateway PHP\r\n";
	$mail_headers .= "MIME-Version: 1.0\r\n";
	$mail_headers .= "Content-Type: multipart/mixed; boundary=\"{$mail_boundary}\"\r\n";
	$mail_body = "This message is in MIME format.\r\n\r\n";
	$mail_body .= "--{$mail_boundary}\r\n";
	$mail_body .= "Content-Type: text/plain; charset=\"iso-8859-1\"\r\n";
	$mail_body .= "Content-Transfer-Encoding:7bit\r\n\r\n";

	$mail_body .= "Here is your CSV file you requested.\r\n\r\n";

	$mail_file = "hostname,ip,hardware address,operating system,hardware type,location,serial number,host status,bandwidth rate,purchase date,po number,warranty date,owning unit,other id,last seen\r\r\n";

	foreach ($hosts as $host) {
		$domain  = $host->domain->name;
		$ip_addr = $host->ip->ip_addr;
		$date = date("m\/d\/Y H:i:s",$host->last_seen);
		if ($ip_addr <= "0") { $ip_addr = "Dynamic"; } 

		//we need to do a str_replace for ,'s do they don't breka the csv
		$host->os 			= str_replace(",","_",$host->os);
		$host->type 			= str_replace(",","_",$host->type);
		$host->location 		= str_replace(",","_",$host->location);
		$host->serial			= str_replace(",","_",$host->serial);
		$host->other_id			= str_replace(",","_",$host->other_id);
		$date 				= str_replace("12/31/1969 18:00:00","never",$date);
		$workgroup			= $host->workgroup;
		$workgroup_name			= str_replace(",","_",$workgroup->name);

		$mail_file .= "$host->name.$domain,$ip_addr,$host->ha,$host->os,$host->type,$host->location,$host->serial,$host->status,$host->bandwidth_rate,$host->purchase_date,$host->po_number,$host->warranty_date,$host->owning_unit,$host->other_id,$date,$workgroup_name\r\r\n";
	}

	$mail_body .= "--{$mail_boundary}\r\n";
	$mail_body .= "Content-Type: application/octet-stream; name=\"hosts.csv\"\r\n";
	$mail_body .= "Content-Transfer-Encoding: quoted-printable\r\n";
	$mail_body .= "Content-Disposition: attachment; filename=\"hosts.csv\"\r\n\r\n";

	$mail_body .= $mail_file ."\r\n\r\n";
	$mail_body .= "--{$mail_boundary}\r\n";

	mail ($email, "CSV Host Report of $zone", $mail_body, $mail_headers);

	} //end host report

	//if the cnames are set send a cname report
	if ($cnames) {
	$mail_body  = "# format for this file is the following\n";
        $mail_body .= "# hostname,cname server\n";

        foreach ($cnames as $cname) {
                $mail_body .= "$cname->hostname,$cname->server\n";
        }

        mail ($email, "CSV Cname Report of $zone", $mail_body,
                "From: $email\r\n" .
                "X-Mailer: NET Mail Gateway PHP/" . phpversion());
	} //end cname report

        //if the mxss are set send a mx report
        if ($mxs) {
        $mail_body  = "# format for this file is the following\n";
        $mail_body .= "# hostname,mx server,priority\n";

        foreach ($mxs as $mx) {
                $mail_body .= "$mx->hostname,$mx->server,$mx->priority\n";
        }

        mail ($email, "CSV MX Record Report of $zone", $mail_body,
                "From: $email\r\n" .
                "X-Mailer: NET Mail Gateway PHP/" . phpversion());

        } //end mx report

        //if there are flos set send a flo report
        if ($flos) {
        $mail_body  = "# format for this file is the following\n";
        $mail_body .= "date,hostname,domain,os,location,hardware type,serial,other_id,ip,in_kb,out_kb,in_packets,out_packets\n";

        foreach ($flos as $flo) {
		$date = date("m\/d\/Y",$flo->date);
		$hostname = $flo->host->name;
		$ip_addr = $flo->host->ip->ip_addr;
		$serial = $flo->host->serial;
		$os = $flo->host->os;
		$location = $flo->host->location;
		$other_id = $flo->host->other_id;
		$domain = $flo->host->domain->name;

		$os 			= str_replace(",","_",$os);
		$type 			= str_replace(",","_",$type);
		$location 		= str_replace(",","_",$location);
		$serial			= str_replace(",","_",$serial);
		$other_id		= str_replace(",","_",$other_id);

                $mail_body .= "$date,$hostname,$domain,$os,$location,$type,$serial,$other_id,$ip_addr,$flo->in_kb,$flo->out_kb,$flo->in_packets,$flo->out_packets\n";
        }

        mail ($email, "CSV Bandwidth Usage Report of $zone", $mail_body,
                "From: $email\r\n" .
                "X-Mailer: NET Mail Gateway PHP/" . phpversion());

        } //end flos report


} //deliver_csv

// This function simply takes a number and puts commas every 3.
function commify($int) {
	global $conf;
	$count = intval(strlen($int)/3);
	if ($count == strlen($int)/3) { $count -= 1; }
	$i=1;
	while ($i <= $count) {
		$int = substr_replace($int, ",",((strlen($int)-$b)-(3 * $i)),0);
		$i++;
		$b++;
		}
	return $int;


} //commify

/*!
	@function admin_only()
	@discussion Checks the users credentials, if they are not an admin
		it shows an access denied sign.  This is mainly for use in docs.
*/
function ensure_admin_credentials($user) {
	global $conf;
	if (!$user->is_admin) {
		$conf['ui']->show_access_denied();
		exit();
	}
}

function date_or_null($format, $date) {
	if ($date != 0) {
		return date($format, $date);
	}
}

?>
